@@ -183,6 +183,13 @@ int xmp_get_error()
 #endif
 }
 
+// the error callback - force throwing exceptions
+static bool _xmp_error_callback(void* context, XMP_ErrorSeverity severity,
+                                XMP_Int32 cause, XMP_StringPtr message)
+{
+    return false;
+}
+
 bool xmp_init()
 {
     RESET_ERROR;
@@ -192,7 +199,9 @@ bool xmp_init()
         // text
         // conversion
         // the one that was done in Exempi with libiconv.
-        return SXMPFiles::Initialize(kXMPFiles_IgnoreLocalText);
+        bool result = SXMPFiles::Initialize(kXMPFiles_IgnoreLocalText);
+        SXMPMeta::SetDefaultErrorCallback(&_xmp_error_callback, nullptr, 1);
+        return result;
     }
     catch (const XMP_Error &e) {
         set_error(e);
